#include "RedChannelAuto.h"
#include "DayLightSensor.h"
#include "RedLightSwitch.h"
#include "IntervalTimer.h"

static RedChannelAuto_State state;

void RedChannelAuto_init(void)
{
    DayLightSensor_init();
    RedLightSwitch_init();
    IntervalTimer_init();

    RedLightSwitch_off();
    state = INACTIVE;
}

RedChannelAuto_State RedChannelAuto_getState(void)
{
    return state;
}

//    INACTIVE
static void transTo_INACTIVE(void)
{
    RedLightSwitch_off();
    state = INACTIVE;
}

//    LIGHT_OFF
static void transTo_LIGHT_OFF(void)
{
    RedLightSwitch_off();
    IntervalTimer_start((uint16_t)1500);
    state = LIGHT_OFF;
}

//    LIGHT_ON
static void transTo_LIGHT_ON(void)
{
    RedLightSwitch_on();
    IntervalTimer_start((uint16_t)500);
    state = LIGHT_ON;
}

void RedChannelAuto_run(void)
{
    switch (state)
    {
        case INACTIVE:
            if (DayLightSensor_isDay())
            {
            }
            else
            {
                transTo_LIGHT_OFF();
            }
            return;

        case LIGHT_OFF:
            if (DayLightSensor_isDay())
            {
                transTo_INACTIVE();
            }
            else
            {
                if (IntervalTimer_isElapsed())
                {
                    transTo_LIGHT_ON();
                }
            }
            return;

        case LIGHT_ON:
            if (DayLightSensor_isDay())
            {
                transTo_INACTIVE();
            }
            else
            {
                if (IntervalTimer_isElapsed())
                {
                    transTo_LIGHT_OFF();
                }
            }
            return;
    }
}
