#include "unity.h"
#include "RedChannelAuto.h"
#include "MockDayLightSensor.h"
#include "MockRedLightSwitch.h"
#include "MockIntervalTimer.h"

void setUp(void)
{
    DayLightSensor_init_Expect();
    RedLightSwitch_init_Expect();
    IntervalTimer_init_Expect();
    RedLightSwitch_off_Expect();

    RedChannelAuto_init();
}

void tearDown(void)
{
}

/*
      .
         .
           .
       .
        .
           INACTIVE
*/
void test_RedChannelAuto_Init(void)
{
    TEST_ASSERT_EQUAL_INT_MESSAGE(INACTIVE, RedChannelAuto_getState(),
                                  "State must be INACTIVE after initialization");
}

/*
    : ,    INACTIVE,   
      .
*/
void test_RedChannelAuto_Keep_State_INACTIVE_During_Day(void)
{
    TEST_ASSERT_EQUAL_INT_MESSAGE(INACTIVE, RedChannelAuto_getState(),
                                  "Initial state expected to be INACTIVE");
    DayLightSensor_isDay_ExpectAndReturn((_Bool)1);

    RedChannelAuto_run();

    TEST_ASSERT_EQUAL_INT_MESSAGE(INACTIVE, RedChannelAuto_getState(),
                                  "State must be INACTIVE during day");
}

/*
     :   INACTIVE  LIGHT_OFF .
    :  = INACTIVE
    :  = LIGHT_OFF,   ,
       1500 .
*/
static void helper_Trans_From_INACTIVE_To_LIGHT_OFF_At_Night(void)
{
    //      INACTIVE
    TEST_ASSERT_EQUAL_INT_MESSAGE(INACTIVE, RedChannelAuto_getState(),
                                  "Initial state expected to be INACTIVE");
    DayLightSensor_isDay_ExpectAndReturn((_Bool)0); //  
    RedLightSwitch_off_Expect();    //     
    IntervalTimer_start_Expect((uint16_t)1500); //     1500 

    RedChannelAuto_run();
}

/*
    : ,    INACTIVE,    
       LIGHT_OFF.
              
       (1500 ).
*/
void test_RedChannelAuto_Trans_From_INACTIVE_To_LIGHT_OFF_At_Night(void)
{
    //     LIGHT_OFF
    helper_Trans_From_INACTIVE_To_LIGHT_OFF_At_Night();

    TEST_ASSERT_EQUAL_INT_MESSAGE(LIGHT_OFF, RedChannelAuto_getState(),
                                  "State expected to be LIGHT_OFF");
}

/*
    : ,    LIGHT_OFF ,   
         .
*/
void test_RedChannelAuto_Keep_State_LIGHT_OFF_At_Night_During_Pause(void)
{
    //     LIGHT_OFF
    helper_Trans_From_INACTIVE_To_LIGHT_OFF_At_Night();
    TEST_ASSERT_EQUAL_INT_MESSAGE(LIGHT_OFF, RedChannelAuto_getState(),
                                  "Initial state expected to be LIGHT_OFF");
    DayLightSensor_isDay_ExpectAndReturn((_Bool)0); //  
    IntervalTimer_isElapsed_ExpectAndReturn((_Bool)0);  //    

    RedChannelAuto_run();

    TEST_ASSERT_EQUAL_INT_MESSAGE(LIGHT_OFF, RedChannelAuto_getState(),
                                  "State still expected to be LIGHT_OFF");
}

/*
    : ,    LIGHT_OFF,    
       INACTIVE.
         .
*/
void test_RedChannelAuto_Trans_From_LIGHT_OFF_To_INACTIVE_At_Day(void)
{
    //     LIGHT_OFF
    helper_Trans_From_INACTIVE_To_LIGHT_OFF_At_Night();
    TEST_ASSERT_EQUAL_INT_MESSAGE(LIGHT_OFF, RedChannelAuto_getState(),
                                  "Initial state expected to be LIGHT_OFF");
    DayLightSensor_isDay_ExpectAndReturn((_Bool)1); //  
    RedLightSwitch_off_Expect();    //     

    RedChannelAuto_run();

    TEST_ASSERT_EQUAL_INT_MESSAGE(INACTIVE, RedChannelAuto_getState(),
                                  "State expected to be INACTIVE");
}

/*
     :   INACTIVE  LIGHT_ON .
    :  = INACTIVE
    :  = LIGHT_ON,   ,
       500 .
*/
static void helper_Trans_From_INACTIVE_To_LIGHT_ON_At_Night(void)
{
    //     LIGHT_OFF
    helper_Trans_From_INACTIVE_To_LIGHT_OFF_At_Night();
    TEST_ASSERT_EQUAL_INT_MESSAGE(LIGHT_OFF, RedChannelAuto_getState(),
                                  "State expected to be LIGHT_OFF");
    DayLightSensor_isDay_ExpectAndReturn((_Bool)0); //  
    IntervalTimer_isElapsed_ExpectAndReturn((_Bool)1);  //    
    RedLightSwitch_on_Expect();    //     
    IntervalTimer_start_Expect((uint16_t)500); //     500 

    RedChannelAuto_run();
}

/*
    : ,    LIGHT_OFF ,   
          LIGHT_ON.
              
      (500 ).
*/
void test_RedChannelAuto_Trans_From_LIGHT_OFF_To_LIGHT_ON_At_Night(void)
{
    helper_Trans_From_INACTIVE_To_LIGHT_ON_At_Night();

    TEST_ASSERT_EQUAL_INT_MESSAGE(LIGHT_ON, RedChannelAuto_getState(),
                                  "State expected to be LIGHT_ON");
}

/*
    : ,    LIGHT_ON ,   
       .
*/
void test_RedChannelAuto_Keep_State_LIGHT_ON_At_Night_During_Flash(void)
{
    //     LIGHT_ON
    helper_Trans_From_INACTIVE_To_LIGHT_ON_At_Night();
    TEST_ASSERT_EQUAL_INT_MESSAGE(LIGHT_ON, RedChannelAuto_getState(),
                                  "Initial state expected to be LIGHT_ON");
    DayLightSensor_isDay_ExpectAndReturn((_Bool)0); //  
    IntervalTimer_isElapsed_ExpectAndReturn((_Bool)0);  //    

    RedChannelAuto_run();

    TEST_ASSERT_EQUAL_INT_MESSAGE(LIGHT_ON, RedChannelAuto_getState(),
                                  "State still expected to beLIGHT_ON");
}


/*
    : ,    LIGHT_ON,    
       INACTIVE.
         .
*/
void test_RedChannelAuto_Trans_From_LIGHT_ON_To_INACTIVE_At_Day(void)
{
    //     LIGHT_ON
    helper_Trans_From_INACTIVE_To_LIGHT_ON_At_Night();
    TEST_ASSERT_EQUAL_INT_MESSAGE(LIGHT_ON, RedChannelAuto_getState(),
                                  "Initial state expected to be LIGHT_ON");
    DayLightSensor_isDay_ExpectAndReturn((_Bool)1); //  
    RedLightSwitch_off_Expect();    //     

    RedChannelAuto_run();

    TEST_ASSERT_EQUAL_INT_MESSAGE(INACTIVE, RedChannelAuto_getState(),
                                  "State expected to be INACTIVE");
}

/*
    : ,    LIGHT_ON ,   
          LIGHT_OFF.
              
       (1500 ).
*/
void test_RedChannelAuto_Trans_From_LIGHT_ON_To_LIGHT_OFF_At_Night(void)
{
    //     LIGHT_ON
    helper_Trans_From_INACTIVE_To_LIGHT_ON_At_Night();
    TEST_ASSERT_EQUAL_INT_MESSAGE(LIGHT_ON, RedChannelAuto_getState(),
                                  "Initial state expected to be LIGHT_ON");
    DayLightSensor_isDay_ExpectAndReturn((_Bool)0); //  
    IntervalTimer_isElapsed_ExpectAndReturn((_Bool)1);  //    
    RedLightSwitch_off_Expect();    //     
    IntervalTimer_start_Expect((uint16_t)1500); //     1500 

    RedChannelAuto_run();

    TEST_ASSERT_EQUAL_INT_MESSAGE(LIGHT_OFF, RedChannelAuto_getState(),
                                  "State expected to be LIGHT_OFF");
}
